# Sample code: Load collection from disk then run 1N identification

# This sample app demonstrates how to use 1N identification. First, an existing collection (created by running enroll_in_database) is loaded from disk.
# Next, 1N identification is run to determine the identity of an anonymous template.
# Note, you must run enroll_in_database before being able to run this sample app. 

import tfsdk
import os
options = tfsdk.ConfigurationOptions()
# Can set configuration options here

# options.dbms = tfsdk.DATABASEMANAGEMENTSYSTEM.NONE # The data will not persist after the application terminates

# Load the collection from an SQLITE database
options.dbms = tfsdk.DATABASEMANAGEMENTSYSTEM.SQLITE

# If you previously enrolled the templates into a PostgreSQL database, then use POSTGRESQL instead
# options.dbms = tfsdk.DATABASEMANAGEMENTSYSTEM.POSTGRESQL

options.smallest_face_height = 40 # https://reference.trueface.ai/cpp/dev/latest/usage/general.html#_CPPv4N8Trueface20ConfigurationOptions18smallestFaceHeightE
options.fr_model = tfsdk.FACIALRECOGNITIONMODEL.TFV5 
# Note, if you do use TFV5, you will need to run the download script in /download_models to obtain the model file

# TODO: If you have a NVIDIA gpu, then enable the enableGPU flag (you will require a GPU specific token for this).
# options.enable_GPU = True 

sdk = tfsdk.SDK(options)

# TODO: export your license token as TRUEFACE_TOKEN environment variable
is_valid = sdk.set_license(os.environ['TRUEFACE_TOKEN'])
if (is_valid == False):
    print("Invalid License Provided")
    print("Be sure to export your license token as TRUEFACE_TOKEN")
    quit()


# Connect to our existing database
res = sdk.create_database_connection("my_database.db")
if (res != tfsdk.ERRORCODE.NO_ERROR):
  print("Unable to create database connection")
  quit()


# ex. If using POSTGRESQL backend...
# res = sdk.create_database_connection("host=localhost port=5432 dbname=my_database user=postgres password=admin")
# if (res != tfsdk.ERRORCODE.NO_ERROR):
#   print("Unable to create database connection")
#   quit()

# Load the existing collection into memory
res = sdk.create_load_collection("my_collection")
if (res != tfsdk.ERRORCODE.NO_ERROR):
    print("Unable to create collection")
    quit()

# Use a new image of Brad Pitt as the probe
res = sdk.set_image("../images/brad_pitt_1.jpg")
if (res != tfsdk.ERRORCODE.NO_ERROR):
    print("Unable to set image")
    quit()

# Extract the feature vector
# We do not need to check the quality of the probe face templates
# We mainly want to ensure that the enrollment templates are high quality, 
# This is less of a concern with probe templates 
res, v1, found_face = sdk.get_largest_face_feature_vector()
if (res != tfsdk.ERRORCODE.NO_ERROR or found_face != True):
    print("No face found in image, unable to generate feature vector")
    quit()

# Run identification query using threshold of 0.4
res, found, candidate = sdk.identify_top_candidate(v1, 0.4)

if found == False or res != tfsdk.ERRORCODE.NO_ERROR:
    print("Unable to find match above threshold")
else:
    print("Match found!")
    print("Identity:", candidate.identity)
    print("Match Probability:", candidate.match_probability)
    print("Similarity:", candidate.similarity_measure)